/*
** GetInfo v1.2
**
** Gerneric WWW info skript
**
** Copyright (c) 6.10.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.09.05
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Must be linked with libwww!
**
** On solaris, link with -L/usr/ucblib -lucb -lnsl
**
** Version history
**
** Version 1.2 - 26.7.96
**	Embed output in MIME multipart message
**
** Version 1.1.1 - 23.8.95
**	Patch to avoid buggy automounter on RZ user pools
**
** Version 1.1 - 12.1.95
**	Move all configurable parameters to separate config file
**
** Version 1.0.1 - 29.10.94
**	Added generic admin command
**
** Version 1.0 - 6.10.94
**	Initial version
*/

char header[]="getinfo v1.2\n(c) 1994,1995,1996 by Andreas Ley\n";

#define	DEBUG	/* define to debug public_html */
#define	RZ	/* define for use on RZ user pools */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#ifdef DEBUG
#include <signal.h>
#endif

extern char *HTGetHostName();
extern int strcasecomp();
extern void HTSetHostName();

#define	TRUE	1		/* This is the naked Truth */
#define	FALSE	0		/* and this is the Light */

#define	HTTPD_CONFIG	"/usr/local/etc/httpd/config/httpd.conf"
#define	GETINFO_CONFIG	"/usr/local/etc/httpd/config/getinfo.conf"

/* Defaults */
char		*userdir="public_html";
char		*welcome="index.html";
char		*master="mailto:webmaster@rz.uni-karlsruhe.de";
char		*reference="http://www.uni-karlsruhe.de/~webmaster/";

struct commands {
	char	*name;
	int	(*command)();
	char	*comment;
	} *cmd;
extern struct commands commands[];

#ifdef DEBUG
int	debug=0;
#endif



/******************************************************************************
** version: show program version
******************************************************************************/

int version(char *pathinfo)
{
	(void)printf(header,pathinfo);
	return(0);
}



/******************************************************************************
** help: list all available commands
******************************************************************************/

int help(char *pathinfo)
{
	for(cmd=commands;cmd->name;cmd++)
		if (cmd->comment&&!(pathinfo&&strcmp(cmd->name,pathinfo)))
			(void)printf("%s - %s\n",cmd->name,cmd->comment);
	return(0);
}



/******************************************************************************
** admin: show server adminstrator
******************************************************************************/

int admin(char *pathinfo)
{
	(void)printf("%s\n",master);
	(void)printf("%s\n",reference);
	return(0);
}



/******************************************************************************
** public_html: list all urls of public user html pages
******************************************************************************/

#include <pwd.h>
static void homepage(struct passwd *pw)
{
	char	fname[MAXPATHLEN],*ptr;

#ifdef DEBUG
	if (debug)
		(void)printf("DEBUG begin homepage(\"%s\") [\"%s\"]\n",pw->pw_name,pw->pw_gecos);
#endif
#ifdef RZ
	(void)sprintf(fname,"/hfs/1/user/%s/%s/%s",pw->pw_name,userdir,welcome);
#else
	(void)sprintf(fname,"%s/%s/%s",pw->pw_dir,userdir,welcome);
#endif
#ifdef DEBUG
	if (debug) {
		(void)printf("DEBUG homepage fname=\"%s\"\n",fname);
		(void)printf("DEBUG access(\"%s\",R_OK)=%d\n",fname,access(fname,R_OK));
	}
#endif
	if (!access(fname,R_OK)) {
		if ((ptr=strchr(pw->pw_gecos,',')))
			*ptr='\0';
		(void)printf("http://%s/~%s/\t%s\t%s\n",HTGetHostName(),pw->pw_name,pw->pw_name,pw->pw_gecos);
	}
}

int public_html(char *pathinfo)
{
	struct passwd	*pw;

	if (pathinfo)
		homepage(getpwnam(pathinfo));
	else
		while ((pw=getpwent()))
			homepage(pw);
	return(0);
}



/******************************************************************************/


void GetConfig(char *fname)
{
	FILE	*src;
	char	line[BUFSIZ],*ptr;

	if ((src=fopen(fname,"r"))) {
		while (fgets(line,sizeof(line),src)) {
			ptr=strchr(line,'\0');
			if (*(ptr-1)=='\n')
				ptr--;
			if (*(ptr-1)=='\r')
				ptr--;
			while (ptr>line &&  *(ptr-1)==' ')
				ptr--;
			*ptr='\0';
			ptr=line;
			while(!strchr(" \t",*ptr))
				ptr++;
			*ptr++='\0';
			while(strchr(" \t",*ptr))
				ptr++;
		
			if (!strcasecomp(line,"HostName"))
				HTSetHostName(ptr);
			if (!strcasecomp(line,"UserDir"))
				userdir=strdup(ptr);
			if (!strcasecomp(line,"Welcome"))
				welcome=strdup(ptr);
			if (!strcasecomp(line,"Master"))
				master=strdup(ptr);
			if (!strcasecomp(line,"Reference"))
				reference=strdup(ptr);
		}
		(void)fclose(src);
	}
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v] [filename...]\n",image);
	exit(1);
}


#ifdef DEBUG
void handler(int sig)
{
	(void)printf("DEBUG caught signal %d\n",sig);
	exit(1);
}
#endif


struct commands commands[] = {
	{ "help", help, "list all available commands" },
	{ "version", version, "show program version" },
	{ "admin", admin, "show server adminstrator" },
	{ "public_html", public_html, "list all urls of public user html pages" },
	{ NULL, NULL, NULL } }, *cmd;

int main(int argc, char *argv[])
{
	int	c,i,retval;
	char	*command,*pathinfo,boundary[33];

#ifdef DEBUG
	int	sig;

	for (sig=1;sig<=32;sig++)
		(void)signal(sig,handler);
#endif

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	GetConfig(HTTPD_CONFIG);
	GetConfig(GETINFO_CONFIG);

	srand(time(NULL));
	for (i=0;i<sizeof(boundary)-1;i++)
		boundary[i]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'()+_,-./:=?"[rand()%74];
	boundary[i]='\0';

	(void)printf("Content-type: multipart/mixed; boundary=\"%s\"\n\n",boundary);

	if (!(command=getenv("PATH_INFO")))
		return(1);
	while (*command=='/')
		command++;
	if ((pathinfo=strchr(command,'/')))
		*pathinfo++='\0';
#ifdef DEBUG
	while (!strcmp(command,"debug")) {
		debug++;
		command=pathinfo;
		if ((pathinfo=strchr(command,'/')))
			*pathinfo++='\0';
	}
#endif
	
	for(cmd=commands;cmd->name;cmd++)
		if (!strcmp(cmd->name,command)) {
#ifdef DEBUG
			if (debug)
				(void)printf("DEBUG begin %s(\"%s\")\n",command,pathinfo);
#endif
			(void)printf("--%s\n\n",boundary);
			retval=(*cmd->command)(pathinfo);
			(void)printf("\n--%s--\n",boundary);
#ifdef DEBUG
			if (debug)
				(void)printf("DEBUG end %s()=%d\n",command,retval);
#endif
			return(retval);
		}

	return(1);
}
